# encoding: utf-8
# frozen_string_literal: true

module Carbon
  module Tacky
    Typed = Struct.new(:value, :type) do
      INT32 = Carbon::Type("Carbon::Int32")
      STRING = Carbon::Type("Carbon::String")
      FLOAT = Carbon::Type("Carbon::Float")

      def self.from(value)
        case value
        when Concrete::Type, Tacky::Typed then value
        when Tacky::Parameter             then new(value, value.type)
        when Tacky::Reference             then fail NotImplementedError
        when Tacky::Block                 then nil
        when ::Integer                    then new(value, INT32)
        when ::Float                      then new(value, FLOAT)
        when ::String, ::Symbol           then new(value, STRING)
        else fail ArgumentError, "Cannot guess type for #{value.class}"
        end
      end
    end
  end
end
